use crate::models::{ExportEventRow, Function};
use actix_http::header::CONTENT_DISPOSITION;
use actix_web::{http::header::ContentDisposition, web, HttpResponse, Responder};
use chrono::{Datelike, NaiveDate};
use rust_xlsxwriter::workbook::Workbook;
use serde::Deserialize;
use sqlx::PgPool;

use crate::{
    models::{Role, User},
    utils::{ApplicationError, DateTimeFormat},
};

#[derive(Deserialize)]
struct ExportQuery {
    start: NaiveDate,
    end: NaiveDate,
    area: Option<i32>,
}

#[derive(PartialEq)]
struct EventExportEntry {
    date: String,
    weekday: String,
    start_time: String,
    end_time: String,
    hours: f32,
    location: String,
    name: String,
    assigned_name: Option<String>,
    assigned_function: Option<String>,
}

fn read(rows: Vec<ExportEventRow>) -> Vec<EventExportEntry> {
    let mut entries = Vec::new();

    for r in rows {
        let date = r
            .start_timestamp
            .format(DateTimeFormat::DayMonthYear.into());
        let weekday = r.start_timestamp.weekday();
        let start_time = r
            .start_timestamp
            .time()
            .format(DateTimeFormat::HourMinute.into());
        let end_time = r
            .end_timestamp
            .time()
            .format(DateTimeFormat::HourMinute.into());
        let hours = (r.end_timestamp - r.start_timestamp).as_seconds_f32() / 3600.0 + 1.0;

        let mut event_new: Vec<EventExportEntry> = Vec::new();

        let mut assigned_posten: i16 = 0;
        let mut assigned_wachhabender = false;
        let mut assigned_fuehrungassistent = false;

        for (assigned_name, assigned_function) in r.assignments {
            match assigned_function {
                Function::Posten => {
                    assigned_posten += 1;
                }
                Function::Wachhabender => {
                    assigned_wachhabender = true;
                }
                Function::Fuehrungsassistent => {
                    assigned_fuehrungassistent = true;
                }
            };

            event_new.push(EventExportEntry {
                date: date.to_string(),
                weekday: weekday.to_string(),
                start_time: start_time.to_string(),
                end_time: end_time.to_string(),
                hours,
                location: r.location_name.to_string(),
                name: r.event_name.to_string(),
                assigned_name: Some(assigned_name.trim().to_string()),
                assigned_function: Some(assigned_function.to_string()),
            });
        }

        if !assigned_wachhabender {
            let function = if r.voluntary_wachhabender {
                "WH"
            } else {
                "BF-WH"
            };

            event_new.push(EventExportEntry {
                date: date.to_string(),
                weekday: weekday.to_string(),
                start_time: start_time.to_string(),
                end_time: end_time.to_string(),
                hours,
                location: r.location_name.to_string(),
                name: r.event_name.to_string(),
                assigned_name: None,
                assigned_function: Some(function.to_string()),
            });
        }

        if !assigned_fuehrungassistent && r.voluntary_fuehrungsassistent {
            event_new.push(EventExportEntry {
                date: date.to_string(),
                weekday: weekday.to_string(),
                start_time: start_time.to_string(),
                end_time: end_time.to_string(),
                hours,
                location: r.location_name.to_string(),
                name: r.event_name.to_string(),
                assigned_name: None,
                assigned_function: Some("FüAss".to_string()),
            });
        }

        if assigned_posten < r.amount_of_posten {
            for _ in 0..(r.amount_of_posten - assigned_posten) {
                event_new.push(EventExportEntry {
                    date: date.to_string(),
                    weekday: weekday.to_string(),
                    start_time: start_time.to_string(),
                    end_time: end_time.to_string(),
                    hours,
                    location: r.location_name.to_string(),
                    name: r.event_name.to_string(),
                    assigned_name: None,
                    assigned_function: Some("PO".to_string()),
                });
            }
        }

        for v in r.vehicles {
            event_new.push(EventExportEntry {
                date: date.to_string(),
                weekday: weekday.to_string(),
                start_time: start_time.to_string(),
                end_time: end_time.to_string(),
                hours,
                location: r.location_name.to_string(),
                name: r.event_name.to_string(),
                assigned_name: Some(v.to_string()),
                assigned_function: Some(String::from("Fzg")),
            });
        }

        entries.append(&mut event_new);
    }

    entries
}

#[actix_web::get("/export/eventsdata")]
pub async fn get(
    pool: web::Data<PgPool>,
    user: web::ReqData<User>,
    query: web::Query<ExportQuery>,
) -> Result<impl Responder, ApplicationError> {
    if user.role != Role::Admin && user.role != Role::AreaManager {
        return Err(ApplicationError::Unauthorized);
    }

    let rows_to_export = ExportEventRow::read(pool.get_ref()).await?;
    let entries = read(rows_to_export);

    let mut workbook = Workbook::new();
    let worksheet = workbook.add_worksheet();
    // let time_format = Format::new();
    // time_format.set_num_format(num_format)

    const HEADER: [&str; 10] = [
        "Datum",
        "Wochentag",
        "Beginn",
        "Ende",
        "Stunden",
        "Ort",
        "VA",
        "Namen",
        "Fkt",
        "Reserve",
    ];
    worksheet.write_row(2, 0, HEADER).unwrap();

    for (i, entry) in entries.iter().enumerate() {
        let i = (i + 3) as u32;

        worksheet.write(i, 0, &entry.date).unwrap();
        worksheet.write(i, 1, &entry.weekday).unwrap();
        worksheet.write(i, 2, &entry.start_time).unwrap();
        worksheet.write(i, 3, &entry.end_time).unwrap();
        worksheet.write(i, 4, entry.hours).unwrap();
        worksheet
            .write(i, 4, format!("{:.1}", entry.hours))
            .unwrap();
        worksheet.write(i, 5, &entry.location).unwrap();
        worksheet.write(i, 6, &entry.name).unwrap();
        worksheet.write(i, 7, entry.assigned_name.as_ref()).unwrap();
        worksheet
            .write(i, 8, entry.assigned_function.as_ref())
            .unwrap();
    }

    worksheet.autofit();

    let buffer = workbook.save_to_buffer().unwrap();

    return Ok(HttpResponse::Ok()
        .content_type("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")
        .insert_header((
            CONTENT_DISPOSITION,
            ContentDisposition::attachment("export.xlsx"),
        ))
        .body(buffer));
}
